using System;
using System.Collections.Generic;
using System.Text;
using Lerch.X10;

namespace Lerch.X10Cmd
{
    class Program
    {
        static void Main(string[] args)
        {
            try
            {
                // Set up tracing
                System.Diagnostics.DelimitedListTraceListener writer = new System.Diagnostics.DelimitedListTraceListener("trace.txt");
                //System.Diagnostics.XmlWriterTraceListener writer = new System.Diagnostics.XmlWriterTraceListener("trace.xml");
                writer.Name = "XMLTRACE";
                writer.TraceOutputOptions = System.Diagnostics.TraceOptions.ThreadId | System.Diagnostics.TraceOptions.Timestamp;
                System.Diagnostics.Trace.Listeners.Add(writer);

                IX10Controller controller = X10ControllerFactory.GetX10Controller("COM1");
                using (X10CommandProcessor processor = new X10CommandProcessor(controller))
                {
                    System.Console.Clear();
                    System.Console.Write("> ");
                    string input = System.Console.ReadLine();
                    while (String.Compare(input, "exit", true) != 0)
                    {
                        bool isCommand = false;
                        // Parse & process input
                        string[] tokens = input.Split(' ');
                        if ((tokens != null) && (tokens.Length >= 3))
                        {
                            try
                            {
                                HouseCode house = (HouseCode)Enum.Parse(typeof(HouseCode), tokens[0], true);
                                UnitCode unit = (UnitCode)Enum.Parse(typeof(UnitCode), tokens[1], true);
                                Commands cmd = (Commands)Enum.Parse(typeof(Commands), tokens[2], true);
                                int level = 0;
                                int repeat = 1;
                                bool toggle = false;
                                if (cmd == Commands.BRIGHTEN || cmd == Commands.DIM && tokens.Length >= 4)
                                {
                                    level = Convert.ToInt32(tokens[3]);
                                    if (tokens.Length >= 5)
                                    {
                                        repeat = Convert.ToInt32(tokens[4]);
                                        if (tokens.Length >= 6)
                                            toggle = Convert.ToBoolean(tokens[5]);
                                    }
                                }
                                else
                                {
                                    if (tokens.Length >= 4)
                                    {
                                        repeat = Convert.ToInt32(tokens[3]);
                                        if (tokens.Length >= 5)
                                            toggle = Convert.ToBoolean(tokens[4]);
                                    }
                                }
                                while (repeat-- > 0)
                                {
                                    X10Command x10cmd = new X10Command(house, unit, cmd, level);
                                    processor.SendCommand(x10cmd);
                                    isCommand = true;
                                    System.Console.WriteLine("Command sent: {0} {1} {2} {3}", x10cmd.HouseCode.ToString(), x10cmd.UnitCode.ToString(), x10cmd.Command.ToString(), x10cmd.Level);
                                    if (toggle)
                                    {
                                        if (cmd == Commands.OFF)
                                            cmd = Commands.ON;
                                        else if (cmd == Commands.ON)
                                            cmd = Commands.OFF;
                                        else if (cmd == Commands.BRIGHTEN)
                                            cmd = Commands.DIM;
                                        else if (cmd == Commands.DIM)
                                            cmd = Commands.BRIGHTEN;
                                    }
                                }
                            }
                            catch (Exception)
                            {
                                System.Console.WriteLine("*** Error parsing input ***");
                            }
                        }
                        else if (String.Compare(input, "clear", true) == 0)
                        {
                            System.Console.Clear();
                            isCommand = true;
                        }

                        if (!isCommand)
                            System.Console.WriteLine("Unrecognized command, type 'exit' to quit.");

                        System.Console.Write("> ");
                        input = System.Console.ReadLine();
                    }
                    System.Console.WriteLine("Exiting X10 command line...");
                }
                controller = null;
            }
            catch (Exception ex)
            {
                System.Console.WriteLine("EXCEPTION (type={0}): {1}{2}STACKTRACE: {2}{3}", ex.GetType().FullName, ex.Message, Environment.NewLine, ex.StackTrace);
            }
        }
    }
}